from gtkmvc.controller import Controller
from gtk import main_quit, ListStore, CellRendererText, TreeViewColumn, \
                Statusbar, ProgressBar, Dialog, TreeRowReference, \
                SELECTION_MULTIPLE, DIALOG_MODAL
from gobject import timeout_add, source_remove
from birne_view import BirneSelectorView
from os import path, environ, stat
from time import sleep
import gobject


class BirneController (Controller):
    """
    """

    def __init__(self, model):
        """Contructor. model will be accessible via the member 'self.model'.
        Registration is also performed."""
        Controller.__init__(self, model)
        self.model.registerObserver(self)

        # This is the current working directory for the file selector.
        # We have to remember this because the selector is instantiated
        # on every click on the add button.
        self.dir = str(environ['HOME'])

        self.timer = None

        pass

    def registerView(self, view):
        Controller.registerView(self, view) # Calls the overloaded method

        # Connects the exiting signal:
        view.get_top_widget().connect("destroy", self.on_quit_clicked)

        #
        self.listStore = ListStore(gobject.TYPE_INT,
                                   gobject.TYPE_STRING,
                                   gobject.TYPE_STRING )
        self.view["CDContent"].set_model(self.listStore)
        self.view["CDContent"].get_selection().set_mode(SELECTION_MULTIPLE)

        renderer = CellRendererText()
        sizeColumn = TreeViewColumn("Size (kiB)", renderer, text=0)
        targetColumn = TreeViewColumn("Target", renderer, text=1)
        sourceColumn = TreeViewColumn("Source", renderer, text=2)
        self.view["CDContent"].append_column(sizeColumn)
        self.view["CDContent"].append_column(targetColumn)
        self.view["CDContent"].append_column(sourceColumn)
        pass

    # Signal (view -> self: hey, the user talks to us!)
    def on_add_clicked(self, button):
        selectorCtrl = BirneSelectorController(self.model, self.dir)
        selector = BirneSelectorView(selectorCtrl)
        pass

    def on_remove_clicked(self, button):
        model, selection = self.view["CDContent"].get_selection().get_selected_rows()
        print selection
        for path in selection:
            print model.get_iter_from_string( str(path) )
        pass

    def on_clear_clicked(self, button):
        self.model.content = {}
        self.model.contentsize = 0
        pass

    def on_quit_clicked(self, button=None):
        self.model.cleanup()
        main_quit()

    def on_burn_clicked(self, button):
        self.view["progressbar"].show()
        self.model.burn()
        pass

    def on_mode_toggled(self, button):
        self.model.mode = button.get_name()
        pass

    def on_speed_value_changed(self, button):
        self.model.speed = button.get_value()

    def on_eject_toggled(self, button):
        self.model.eject = button.get_active()
        pass

    def on_simulate_toggled(self, button):
        self.model.simulate = button.get_active()
        pass

    def on_create_iso_clicked(self, button):
        self.model.burn_files(burn_files = False)
        pass

###

    # Observable notifications (model -> self: hey, I am morphing!)

    def property_speed_change_notification(self, model, old, new):
        """  """
        pass

    def property_eject_change_notification(self, model, old, new):
        """  """
        pass

    def property_simulate_change_notification(self, model, old, new):
        """  """
        pass

    def property_content_change_notification(self, model, old, new):
        """ Rebuilds the whole CDContent TreeView. """
        #self.model.status = "C: model property content changed: %s elements" % len(new)
        self.listStore.clear()
        if len(new) > 0:
            sorted_targets = self.model.content.keys()
            sorted_targets.sort()
            for target in sorted_targets:
                source, size = self.model.content[target]
                size /= 1024
                self.listStore.append( (size, target, source) )
        pass

    def property_mode_change_notification(self, model, old, new):
        """  """
        if new == "copy":
            self.view["contentButtonBox"].foreach( lambda w: w.set_sensitive(False) )
            self.view["contentScrolledWindow"].set_sensitive(False)
        elif old == "copy":
            self.view["contentButtonBox"].foreach( lambda w: w.set_sensitive(True) )
            self.view["contentScrolledWindow"].set_sensitive(True)
        pass

    def property_contentsize_change_notification(self, model, old, new):
        """  """
        displaysize = str ( new / 1024 / 1024 ) + " MiB"
        self.view["size"].set_label(displaysize)
        pass

    def property_status_change_notification(self, model, old, new):
        id = self.view["statusbar"].get_context_id("property")
        if new == 'IDLE':
            self.view["interactiveBox"].foreach( lambda w: w.set_sensitive(True) )
            self.view["burn"].set_sensitive(True)
            if self.timer: source_remove(self.timer); self.timer = None
            self.view["progressbar"].set_fraction(0.0)
            self.view["progressbar"].set_text('')
            message = 'Ready.'
        elif new == 'BURNING':
            self.view["interactiveBox"].foreach( lambda w: w.set_sensitive(False) )
            self.view["burn"].set_sensitive(False)
            message = 'Burning...'
        elif new == 'FIXATING':
            self.timer = timeout_add(100, self.show_activity)
            self.view["burn"].set_sensitive(False)
            message = 'Fixating...'
            pass
        elif new == 'READING':
            self.timer = timeout_add(100, self.show_activity)
            self.view["burn"].set_sensitive(False)
            message = ''
            pass
        elif new == 'MKISOFS':
            self.view["interactiveBox"].foreach( lambda w: w.set_sensitive(False) )
            self.view["burn"].set_sensitive(False)
            message = 'Creating ISO file system...'
            pass
        pass
        self.view["statusbar"].push(id, message)

    def property_progress_change_notification(self, model, old, new):
        current, total = [int(s) for s in new]
        fraction = current * 1.0 / total
        if fraction <= 1:
            self.view["progressbar"].set_fraction(fraction)
            self.view["progressbar"].set_text("%s / %s MB" % (current, total) )

    def property_discfree_change_notification(self, model, old, new):
        displaysize = str ( new / 1024 / 1024 ) + " MiB"
        self.view["discfree"].set_text(displaysize);
        pass

    def show_activity(self):
        self.view["progressbar"].pulse()
        return True


    pass # end of class

class BirneSelectorController(Controller):

    def __init__(self, model, dir=str(environ['HOME'])):
        """Contructor. model will be accessible via the member 'self.model'.
        Registration is also performed."""
        Controller.__init__(self, model)
        self.model.registerObserver(self)
        self.dir = dir
        pass

    def registerView(self, view):
        Controller.registerView(self, view) # Calls the overloaded method

        if path.isdir(self.dir):
            self.view["mainWindow"].set_current_folder(self.dir)
        pass

    def on_close_clicked(self, button):
        self.view.get_top_widget().destroy()
        pass

    def on_select_clicked(self, button):
        new_files = self.view["mainWindow"].get_filenames()
        strip_folder = self.view["mainWindow"].get_current_folder()
        is_relative = self.view["rel_paths"].get_active()
        self.model.add_files(new_files, is_relative, strip_folder)
        #print new_files
        pass

    def on_select_and_close_clicked(self, button):
        self.on_select_clicked(button)
        self.on_close_clicked(button)

    pass

class BirneDialogController(Controller):
    def __init__(slef, model):
        Controller.__init__(self, model, thread=None)
        self.model.registerObserver(self)
        pass

    def on_cancel_clicked(self, button):
        pass

# vim: expandtab ts=4
